@@ -4,12 +4,17 @@ import json
 from io import open, StringIO, BytesIO, TextIOWrapper
 from collections import OrderedDict
 import dateutil
+import requests
+import urllib
 
 import pandas as pd
 import pyarrow.parquet as pq
 from chardet.universaldetector import UniversalDetector
 
 from .filepath_or_buffer import FileOrBufferHandler
+from .. import dp_logging
+
+logger = dp_logging.get_child_logger(__name__)
 
 
 def data_generator(data_list):
@@ -333,7 +338,7 @@ def read_parquet_df(file_path, selected_columns=None, read_in_string=False):
     return data, original_df_dtypes
 
 
-def read_text_as_list_of_strs(file_path):
+def read_text_as_list_of_strs(file_path, encoding=None):
     """
     Returns a list of strings relative to the chunk size. Each line is 1 chunk.
     
@@ -342,7 +347,9 @@ def read_text_as_list_of_strs(file_path):
     :return:
     :rtype: list(str)
     """
-    with open(file_path, encoding="utf-8") as input_file:
+    if encoding is None:
+        encoding = detect_file_encoding(file_path)
+    with FileOrBufferHandler(file_path, encoding=encoding) as input_file:
         data = list(input_file)
     return data
 
@@ -382,7 +389,8 @@ def detect_file_encoding(file_path, buffer_size=1024, max_lines=20):
             with FileOrBufferHandler(file_path, encoding=encoding) as input_file:
                 input_file.read(1024*1024)
                 return True
-        except: return False
+        except:
+            return False
 
     if not _decode_is_valid(encoding):
         try:
@@ -395,8 +403,10 @@ def detect_file_encoding(file_path, buffer_size=1024, max_lines=20):
                                         chunk_size=512, threshold=0.2,
                                         cp_isolation=None, cp_exclusion=None,
                                         preemptive_behaviour=True, explain=False)
-                result = result.best().first()
-            if result: encoding = result.encoding
+                result = result.best()
+            if result:
+                if result.first():
+                    encoding = result.first().encoding
 
             # Try again with full sample
             if not _decode_is_valid(encoding): 
@@ -406,11 +416,14 @@ def detect_file_encoding(file_path, buffer_size=1024, max_lines=20):
                                             chunk_size=buffer_size, threshold=0.2,
                                             cp_isolation=None, cp_exclusion=None,
                                             preemptive_behaviour=True, explain=False)
-                    result = result.best().first()
-                if result: encoding = result.encoding
+                    result = result.best()
+                if result:
+                    if result.first():
+                        encoding = result.first().encoding
 
         except:
-            print("Install charset_normalizer for improved file encoding detection")
+            logger.info("Install charset_normalizer for improved file "
+                        "encoding detection")
 
     # If no encoding is still found, default to utf-8
     if not encoding:
@@ -603,3 +616,57 @@ def is_stream_buffer(filepath_or_buffer):
     if isinstance(filepath_or_buffer, (StringIO, BytesIO)):
         return True
     return False
+
+def is_valid_url(url_as_string):
+    """
+    Determines whether a given string is a valid URL
+
+    :param url_as_string: string to be tested if URL 
+    :type url_as_string: str
+    :return: true if string is a valid URL
+    :rtype: boolean
+    """
+    if not isinstance(url_as_string, str):
+        return False
+
+    result = urllib.parse.urlparse(url_as_string)
+    # this is the minimum characteristics needed for a valid URL
+    return all([result.scheme, result.netloc])
+
+
+def url_to_bytes(url_as_string, options):
+    """
+    Reads in URL and converts it to a byte stream
+
+    :param url_as_string: string to read as URL 
+    :type url_as_string: str
+    :param options: options for the url 
+    :type options: dict
+    :return: BytesIO stream of data downloaded from URL
+    :rtype: BytesIO stream
+    """
+    stream = BytesIO()
+
+    verify_url = True
+    if 'verify_ssl' in options:
+        verify_url = options['verify_ssl']
+
+    with requests.get(url_as_string, stream=True, verify=verify_url) as url:
+        url.raise_for_status()
+        if 'Content-length' in url.headers \
+                and int(url.headers['Content-length']) >= 1024 ** 3:
+
+            raise ValueError('The downloaded file from the url may not be larger than 1GB')
+
+        total_bytes = 0
+        c_size = 8192
+
+        for chunk in url.iter_content(chunk_size=c_size):
+            stream.write(chunk)
+            total_bytes += c_size
+
+            if total_bytes > 1024 ** 3:
+                raise ValueError('The downloaded file from the url may not be larger than 1GB')
+
+    stream.seek(0)
+    return stream
