@@ -8,6 +8,9 @@ import numpy as np
 
 from .base_model import BaseModel
 from .base_model import AutoSubRegistrationMeta
+from .. import dp_logging
+
+logger = dp_logging.get_child_logger(__name__)
 
 
 class RegexModel(BaseModel, metaclass=AutoSubRegistrationMeta):
@@ -227,12 +230,11 @@ class RegexModel(BaseModel, metaclass=AutoSubRegistrationMeta):
                         pred[indices[0]:indices[1], entity_id] = 1
             if verbose:
                 sys.stdout.flush()
-                sys.stdout.write(
-                    "\rData Samples Processed: {:d}   ".format(i))
+                sys.stdout.write("\rData Samples Processed: {:d}   ".format(i))
             predictions[i] = pred
 
         if verbose:
-            print()
+            logger.info("\rData Samples Processed: {:d}   ".format(i))
 
         # Trim array size to number of samples
         if len(predictions) > i+1:
