@@ -7,6 +7,7 @@ respective parameters.
 from __future__ import print_function
 from __future__ import division
 
+import scipy.stats
 from future.utils import with_metaclass
 import copy
 import abc
@@ -51,6 +52,7 @@ class NumericStatsMixin(with_metaclass(abc.ABCMeta, object)):
                              "of type NumericalOptions.")
         self.min = None
         self.max = None
+        self._top_k_modes = 5 # By default, return at max 5 modes
         self.sum = 0
         self._biased_variance = np.nan
         self._biased_skewness = np.nan
@@ -67,6 +69,7 @@ class NumericStatsMixin(with_metaclass(abc.ABCMeta, object)):
         self.num_negatives = 0
         if options:
             self.bias_correction = options.bias_correction.is_enabled
+            self._top_k_modes = options.mode.top_k_modes
             bin_count_or_method = \
                 options.histogram_and_quantiles.bin_count_or_method
             if isinstance(bin_count_or_method, str):
@@ -247,6 +250,33 @@ class NumericStatsMixin(with_metaclass(abc.ABCMeta, object)):
         if "num_negatives" in self.__calculations.keys():
             self.num_negatives = other1.num_negatives + other2.num_negatives
 
+        # Merge max k mode count
+        self._top_k_modes = max(other1._top_k_modes, other2._top_k_modes)
+
+    def profile(self):
+        """
+        Property for profile. Returns the profile of the column.
+        :return:
+        """
+        profile = dict(
+            min=self.np_type_to_type(self.min),
+            max=self.np_type_to_type(self.max),
+            mode=self.np_type_to_type(self.mode),
+            sum=self.np_type_to_type(self.sum),
+            mean=self.np_type_to_type(self.mean),
+            variance=self.np_type_to_type(self.variance),
+            stddev=self.np_type_to_type(self.stddev),
+            skewness=self.np_type_to_type(self.skewness),
+            kurtosis=self.np_type_to_type(self.kurtosis),
+            histogram=self._get_best_histogram_for_profile(),
+            quantiles=self.quantiles,
+            num_zeros=self.np_type_to_type(self.num_zeros),
+            num_negatives=self.np_type_to_type(self.num_negatives),
+            times=self.times,
+        )
+
+        return profile
+
     def diff(self, other_profile, options=None):
         """
         Finds the differences for several numerical stats.
@@ -271,6 +301,9 @@ class NumericStatsMixin(with_metaclass(abc.ABCMeta, object)):
                                                    other_profile.variance),
             "stddev": utils.find_diff_of_numbers(self.stddev,
                                                  other_profile.stddev),
+            "t-test": self._perform_t_test(self.mean, self.variance, self.match_count,
+                                           other_profile.mean, other_profile.variance,
+                                           other_profile.match_count)
         }
         return differences
 
@@ -280,6 +313,16 @@ class NumericStatsMixin(with_metaclass(abc.ABCMeta, object)):
             return 0
         return float(self.sum) / self.match_count
 
+    @property
+    def mode(self):
+        """
+        Finds an estimate for the mode(s) of the data.
+
+        :return: the mode(s) of the data
+        :rtype: list(float)
+        """
+        return self._estimate_mode_from_histogram()
+
     @property
     def variance(self):
         return self._biased_variance if not self.bias_correction \
@@ -307,6 +350,52 @@ class NumericStatsMixin(with_metaclass(abc.ABCMeta, object)):
                     self.match_count,
                     self._biased_kurtosis)
 
+    @staticmethod
+    def _perform_t_test(mean1, var1, n1,
+                        mean2, var2, n2):
+        results = {
+            't-statistic': None,
+            'conservative': {
+                'df': None,
+                'p-value': None
+            },
+            'welch': {
+                'df': None,
+                'p-value': None
+            }
+        }
+
+        invalid_stats = False
+        if n1 <= 1 or n2 <= 1:
+            warnings.warn("Insufficient sample size. "
+                          "T-test cannot be performed.", RuntimeWarning)
+            invalid_stats = True
+        if np.isnan([mean1, mean2, var1, var2]).any() or \
+                None in [mean1, mean2, var1, var2]:
+            warnings.warn("Null value(s) found in mean and/or variance values. "
+                          "T-test cannot be performed.", RuntimeWarning)
+            invalid_stats = True
+        if invalid_stats:
+            return results
+
+        s_delta = var1/n1 + var2/n2
+        t = (mean1 - mean2) / np.sqrt(s_delta)
+        conservative_df = min(n1, n2) - 1
+        welch_df = s_delta ** 2 / ((var1 / n1) ** 2 /
+                                   (n1 - 1) + (var2 / n2) ** 2 / (n2 - 1))
+        results['t-statistic'] = t
+        results['conservative']['df'] = conservative_df
+        results['welch']['df'] = welch_df
+        
+        conservative_t = scipy.stats.t(conservative_df)
+        conservative_p_val = (1 - conservative_t.cdf(abs(t))) * 2
+        welch_t = scipy.stats.t(welch_df)
+        welch_p_val = (1 - welch_t.cdf(abs(t))) * 2
+
+        results['conservative']['p-value'] = conservative_p_val
+        results['welch']['p-value'] = welch_p_val
+        return results
+
     def _update_variance(self, batch_mean, batch_var, batch_count):
         """
         Calculate the combined biased variance of the current values and new dataset.
@@ -501,6 +590,40 @@ class NumericStatsMixin(with_metaclass(abc.ABCMeta, object)):
                    (biased_kurtosis + 3) - 3 * (match_count - 1))
         return kurtosis
 
+    def _estimate_mode_from_histogram(self):
+        """
+        Estimates the mode of the current data using the
+        histogram. If there are multiple modes, returns
+        K of them (where K is defined in options given, but
+        5 by default)
+
+        :return: The estimated mode of the histogram
+        :rtype: list(float)
+        """
+        if not self._has_histogram:
+            return np.nan
+
+        bin_counts = self._stored_histogram['histogram']['bin_counts']
+        bin_edges = self._stored_histogram['histogram']['bin_edges']
+
+        # Get the K bin(s) with the highest frequency (one-pass):
+        cur_max = -1
+        highest_idxs = []
+        count = 0
+        for i in range(0, len(bin_counts)):
+            if bin_counts[i] > cur_max:
+                # If a new maximum frequency is found, reset the mode counts
+                highest_idxs = [i]
+                cur_max = bin_counts[i]
+                count = 1
+            elif bin_counts[i] == cur_max and count < self._top_k_modes:
+                highest_idxs.append(i)
+                count += 1
+        highest_idxs = np.array(highest_idxs)
+
+        mode = (bin_edges[highest_idxs] + bin_edges[highest_idxs + 1]) / 2
+        return mode.tolist()
+
     def _estimate_stats_from_histogram(self):
         # test estimated mean and var
         bin_counts = self._stored_histogram['histogram']['bin_counts']
@@ -851,6 +974,7 @@ class NumericStatsMixin(with_metaclass(abc.ABCMeta, object)):
         :return: List of corresponding values for which the percentage of values
             in the distribution fall before each percentage
         """
+        percentiles = np.array(percentiles)
         bin_counts = self._stored_histogram['histogram']['bin_counts']
         bin_edges = self._stored_histogram['histogram']['bin_edges']
 
@@ -872,10 +996,10 @@ class NumericStatsMixin(with_metaclass(abc.ABCMeta, object)):
         cumsum_bin_counts = np.append([0], cumsum_bin_counts)
 
         quantiles = np.interp(percentiles / 100,
-                              cumsum_bin_counts, bin_edges).tolist()
+                              cumsum_bin_counts, bin_edges)
         if median_value:
-            quantiles[499] = median_value
-        return quantiles
+            quantiles[percentiles == 50] = median_value
+        return quantiles.tolist()
 
     def _get_quantiles(self):
         """
@@ -908,10 +1032,11 @@ class NumericStatsMixin(with_metaclass(abc.ABCMeta, object)):
                                      "biased_kurtosis": self._biased_kurtosis}
         subset_properties = copy.deepcopy(profile)
         df_series_clean = df_series_clean.astype(float)
-        super(NumericStatsMixin, self)._perform_property_calcs(self.__calculations,
-                                     df_series=df_series_clean,
-                                     prev_dependent_properties=prev_dependent_properties,
-                                     subset_properties=subset_properties)
+        super(NumericStatsMixin, self)._perform_property_calcs(
+            self.__calculations,
+            df_series=df_series_clean,
+            prev_dependent_properties=prev_dependent_properties,
+            subset_properties=subset_properties)
         if len(self._batch_history) == 5:
             self._batch_history.pop(0)
         self._batch_history.append(subset_properties)
