@@ -15,6 +15,7 @@ from collections import OrderedDict, defaultdict
 import warnings
 import pickle
 from datetime import datetime
+import logging
 
 import pandas as pd
 import numpy as np
@@ -27,6 +28,9 @@ from ..labelers.data_labelers import DataLabeler
 from .helpers.report_helpers import calculate_quantiles, _prepare_report
 from .profiler_options import ProfilerOptions, StructuredOptions, \
     UnstructuredOptions
+from .. import dp_logging
+
+logger = dp_logging.get_child_logger(__name__)
 
 
 class StructuredColProfiler(object):
@@ -58,7 +62,6 @@ class StructuredColProfiler(object):
         self._min_true_samples = min_true_samples
         if self._min_true_samples is None:
             self._min_true_samples = 0
-
         self.sample_size = 0
         self.sample = list()
         self.null_count = 0
@@ -70,6 +73,20 @@ class StructuredColProfiler(object):
         self._last_batch_size = None
         self.profiles = {}
 
+        NO_FLAG = 0
+        self._null_values = {
+            "": NO_FLAG,
+            "nan": re.IGNORECASE,
+            "none": re.IGNORECASE,
+            "null": re.IGNORECASE,
+            "  *": NO_FLAG,
+            "--*": NO_FLAG,
+            "__*": NO_FLAG,
+        }
+        if options:
+            if options.null_values is not None:
+                self._null_values = options.null_values
+
         if df_series is not None and len(df_series) > 0:
 
             if not sample_size:
@@ -82,7 +99,9 @@ class StructuredColProfiler(object):
             clean_sampled_df, base_stats = \
                 self.clean_data_and_get_base_stats(
                     df_series=df_series, sample_size=sample_size,
-                    min_true_samples=self._min_true_samples, sample_ids=sample_ids)
+                    null_values=self._null_values,
+                    min_true_samples=self._min_true_samples,
+                    sample_ids=sample_ids)
             self.update_column_profilers(clean_sampled_df, pool)
             self._update_base_stats(base_stats)
 
@@ -183,6 +202,69 @@ class StructuredColProfiler(object):
             )
         return merged_profile
 
+    def diff(self, other_profile, options=None):
+        """
+        Finds the difference between 2 StructuredCols and returns the report
+
+        :param other: Structured col finding the difference with this one.
+        :type other: StructuredColProfiler
+        :param options: options to change results of the difference
+        :type options: dict
+        :return: difference of the structured column
+        :rtype: dict
+        """
+        unordered_profile = dict()
+        for key, profile in self.profiles.items():
+            if key in other_profile.profiles:
+                comp_diff = self.profiles[key].diff(other_profile.profiles[key],
+                                                    options=options)
+                utils.dict_merge(unordered_profile, comp_diff)
+
+        name = self.name
+        if isinstance(self.name, np.integer):
+            name = int(name)
+
+        unordered_profile.update({
+            "column_name": name,
+        })
+
+        unordered_profile["statistics"].update({
+            "sample_size": utils.find_diff_of_numbers(
+                self.sample_size, other_profile.sample_size),
+            "null_count": utils.find_diff_of_numbers(
+                self.null_count, other_profile.null_count),
+            "null_types": utils.find_diff_of_lists_and_sets(
+                self.null_types, other_profile.null_types),
+            "null_types_index": utils.find_diff_of_dicts_with_diff_keys(
+                self.null_types_index, other_profile.null_types_index),
+        })
+
+        if unordered_profile.get("data_type", None) is not None:
+            unordered_profile["statistics"].update({
+                "data_type_representation":
+                    unordered_profile["data_type_representation"]
+            })
+
+        dict_order = [
+            "column_name",
+            "data_type",
+            "data_label",
+            "categorical",
+            "order",
+            "statistics",
+        ]
+        profile = OrderedDict()
+        if 'data_label_profile' not in self.profiles or\
+                'data_label_profile' not in other_profile.profiles:
+            dict_order.remove("data_label")
+        for key in dict_order:
+            try:
+                profile[key] = unordered_profile[key]
+            except KeyError as e:
+                profile[key] = None
+
+        return profile
+    
     @property
     def profile(self):
         unordered_profile = dict()
@@ -302,6 +384,7 @@ class StructuredColProfiler(object):
         clean_sampled_df, base_stats = \
             self.clean_data_and_get_base_stats(
                 df_series=df_series, sample_size=sample_size,
+                null_values=self._null_values,
                 min_true_samples=min_true_samples, sample_ids=sample_ids)
 
         self._update_base_stats(base_stats)
@@ -324,7 +407,7 @@ class StructuredColProfiler(object):
     # TODO: flag column name with null values and potentially return row
     #  index number in the error as well
     @staticmethod
-    def clean_data_and_get_base_stats(df_series, sample_size,
+    def clean_data_and_get_base_stats(df_series, sample_size, null_values=None,
                                       min_true_samples=None,
                                       sample_ids=None):
         """
@@ -335,6 +418,10 @@ class StructuredColProfiler(object):
         :type df_series: pandas.core.series.Series
         :param sample_size: Number of samples to use in generating the profile
         :type sample_size: int
+        :param null_values: Dictionary mapping null values to regex flag where
+            the key represents the null value to remove from the data and the
+            flag represents the regex flag to apply
+        :type null_values: dict[str, re.FLAG]
         :param min_true_samples: Minimum number of samples required for the
             profiler
         :type min_true_samples: int
@@ -344,20 +431,13 @@ class StructuredColProfiler(object):
             parameters
         :rtype: pd.Series, dict
         """
-        NO_FLAG = 0
-        null_values_and_flags = {
-            "": NO_FLAG,
-            "nan": re.IGNORECASE,
-            "none": re.IGNORECASE,
-            "null": re.IGNORECASE,
-            "  *": NO_FLAG,
-            "--*": NO_FLAG,
-            "__*": NO_FLAG,
-        }
 
         if min_true_samples is None:
             min_true_samples = 0
 
+        if null_values is None:
+            null_values = dict()
+
         len_df = len(df_series)
         if not len_df:
             return df_series, {
@@ -397,7 +477,7 @@ class StructuredColProfiler(object):
         na_columns = dict()
         true_sample_list = set()
         total_sample_size = 0
-        query = '|'.join(null_values_and_flags.keys())
+        query = '|'.join(null_values.keys())
         regex = f"^(?:{(query)})$"
         for chunked_sample_ids in sample_ind_generator:
             total_sample_size += len(chunked_sample_ids)
@@ -468,6 +548,9 @@ class BaseProfiler(object):
         :type options: ProfilerOptions Object
         :return: Profiler
         """
+        if min_true_samples is not None and not isinstance(min_true_samples, int):
+            raise ValueError('`min_true_samples` must be an integer or `None`.')
+        
         if self._default_labeler_type is None:
             raise ValueError('`_default_labeler_type` must be set when '
                              'overriding `BaseProfiler`.')
@@ -487,6 +570,7 @@ class BaseProfiler(object):
         self._samples_per_update = samples_per_update
         self._min_true_samples = min_true_samples
         self.total_samples = 0
+        self.times = defaultdict(float)
 
         # TODO: allow set via options
         self._sampling_ratio = 0.2
@@ -547,8 +631,34 @@ class BaseProfiler(object):
 
         merged_profile.total_samples = self.total_samples + other.total_samples
 
+        merged_profile.times = utils.add_nested_dictionaries(self.times,
+                                                             other.times)
+
         return merged_profile
 
+    def diff(self, other_profile, options=None):
+        """
+        Finds the difference of two profiles
+        :param other: profile being added to this one.
+        :type other: BaseProfiler
+        :return: diff of the two profiles
+        :rtype: dict
+        """
+        if type(other_profile) is not type(self):
+            raise TypeError('`{}` and `{}` are not of the same profiler type.'.
+                            format(type(self).__name__, 
+                                   type(other_profile).__name__))
+
+        diff_profile = OrderedDict([
+            ("global_stats", {
+                "file_type": utils.find_diff_of_strings_and_bools(
+                    self.file_type, other_profile.file_type),
+                "encoding": utils.find_diff_of_strings_and_bools(
+                    self.encoding, other_profile.encoding),
+            })])
+
+        return diff_profile
+
     def _get_sample_size(self, data):
         """
         Determines the minimum sampling size for profiling the dataset.
@@ -630,6 +740,10 @@ class BaseProfiler(object):
         encoding = None
         file_type = None
 
+        if min_true_samples is not None \
+                and not isinstance(min_true_samples, int):
+            raise ValueError('`min_true_samples` must be an integer or `None`.')
+
         if isinstance(data, data_readers.base_data.BaseData):
             encoding = data.file_encoding
             file_type = data.data_type
@@ -884,6 +998,7 @@ class UnstructuredProfiler(BaseProfiler):
 
         # Unstructured specific properties
         self._empty_line_count = 0
+        self.memory_size = 0
         self.sample = []
 
         if data is not None:
@@ -910,6 +1025,7 @@ class UnstructuredProfiler(BaseProfiler):
         # unstruct specific property merging
         merged_profile._empty_line_count = (
                 self._empty_line_count + other._empty_line_count)
+        merged_profile.memory_size = self.memory_size + other.memory_size
         samples = list(dict.fromkeys(self.sample + other.sample))
         merged_profile.sample = random.sample(list(samples),
                                               min(len(samples), 5))
@@ -918,6 +1034,33 @@ class UnstructuredProfiler(BaseProfiler):
         merged_profile._profile = self._profile + other._profile
 
         return merged_profile
+    
+    def diff(self, other_profile, options=None):
+        """
+        Finds the difference between 2 unstuctured profiles and returns the 
+        report.
+
+        :param other: profile finding the difference with this one.
+        :type other: UnstructuredProfiler
+        :param options: options to impact the results of the diff
+        :type options: dict
+        :return: difference of the profiles
+        :rtype: dict
+        """
+        report = super().diff(other_profile, options)
+
+        report["global_stats"].update({
+                "samples_used": utils.find_diff_of_numbers(
+                    self.total_samples, other_profile.total_samples),
+                "empty_line_count": utils.find_diff_of_numbers(
+                    self._empty_line_count, other_profile._empty_line_count),
+                "memory_size": utils.find_diff_of_numbers(
+                    self.memory_size, other_profile.memory_size),
+            })
+
+        report["data_stats"] = self._profile.diff(other_profile._profile, 
+                                                  options=options)
+        return _prepare_report(report)
 
     def _update_base_stats(self, base_stats):
         """
@@ -931,6 +1074,7 @@ class UnstructuredProfiler(BaseProfiler):
         self.total_samples += base_stats["sample_size"]
         self.sample = base_stats["sample"]
         self._empty_line_count += base_stats["empty_line_count"]
+        self.memory_size += base_stats["memory_size"]
 
     def report(self, report_options=None):
         """
@@ -965,16 +1109,17 @@ class UnstructuredProfiler(BaseProfiler):
                 "samples_used": self.total_samples,
                 "empty_line_count": self._empty_line_count,
                 "file_type": self.file_type,
-                "encoding": self.encoding
+                "encoding": self.encoding,
+                "memory_size": self.memory_size,
+                "times": self.times,
             }),
             ("data_stats", OrderedDict()),
         ])
-
         report["data_stats"] = self._profile.profile
         return _prepare_report(report, output_format, omit_keys)
 
-    @staticmethod
-    def _clean_data_and_get_base_stats(data, sample_size,
+    @utils.method_timeit(name="clean_and_base_stats")
+    def _clean_data_and_get_base_stats(self, data, sample_size,
                                        min_true_samples=None):
         """
         Identify empty rows and return a cleaned version of text data without
@@ -997,12 +1142,16 @@ class UnstructuredProfiler(BaseProfiler):
         len_data = len(data)
         if not len_data:
             return data, {
-                "sample_size": 0, "empty_line_count": dict(), "sample": [],
+                "sample_size": 0, "empty_line_count": dict(),
+                "sample": [], "memory_size": 0
             }
 
         # ensure all data are of type str
         data = data.apply(str)
 
+        # get memory size
+        base_stats = {"memory_size": utils.get_memory_size(data, unit='M')}
+
         # Setup sample generator
         sample_ind_generator = utils.shuffle_in_chunks(
             len_data, chunk_size=sample_size)
@@ -1038,12 +1187,14 @@ class UnstructuredProfiler(BaseProfiler):
         data = data.loc[true_sample_list]
         total_empty = total_sample_size - len(true_sample_list)
 
-        base_stats = {
-            "sample_size": total_sample_size,
-            "empty_line_count": total_empty,
-            "sample": random.sample(list(data.values),
-                                    min(len(data), 5)),
-        }
+        base_stats.update(
+            {
+                "sample_size": total_sample_size,
+                "empty_line_count": total_empty,
+                "sample": random.sample(list(data.values),
+                                        min(len(data), 5)),
+            }
+        )
 
         return data, base_stats
 
@@ -1077,7 +1228,7 @@ class UnstructuredProfiler(BaseProfiler):
 
         # Format the data
         notification_str = "Finding the empty lines in the data..."
-        print(notification_str)
+        logger.info(notification_str)
         data, base_stats = self._clean_data_and_get_base_stats(
             data, sample_size, min_true_samples)
         self._update_base_stats(base_stats)
@@ -1089,7 +1240,7 @@ class UnstructuredProfiler(BaseProfiler):
 
         # process the text data
         notification_str = "Calculating the statistics... "
-        print(notification_str)
+        logger.info(notification_str)
         pool = None
         if self._profile is None:
             self._profile = UnstructuredCompiler(data, options=self.options,
@@ -1114,8 +1265,10 @@ class UnstructuredProfiler(BaseProfiler):
             "_samples_per_update": self._samples_per_update,
             "_min_true_samples": self._min_true_samples,
             "_empty_line_count": self._empty_line_count,
+            "memory_size": self.memory_size,
             "options": self.options,
-            "_profile": self.profile
+            "_profile": self.profile,
+            "times": self.times,
         }
         self._save_helper(filepath, data_dict)
 
@@ -1216,10 +1369,73 @@ class StructuredProfiler(BaseProfiler):
         merged_profile._col_name_to_idx = copy.deepcopy(self._col_name_to_idx)
 
         # merge correlation
-        merged_profile.correlation_matrix = self._merge_correlation(other)
+        if (self.options.correlation.is_enabled
+                and other.options.correlation.is_enabled):
+            merged_profile.correlation_matrix = self._merge_correlation(other)
 
         return merged_profile
 
+    def diff(self, other_profile, options=None):
+        """
+        Finds the difference between 2 Profiles and returns the report
+
+        :param other: profile finding the difference with this one
+        :type other: StructuredProfiler
+        :param options: options to change results of the difference
+        :type options: dict
+        :return: difference of the profiles
+        :rtype: dict
+        """
+        report = super().diff( other_profile, options)
+        report["global_stats"].update({
+                "samples_used": utils.find_diff_of_numbers(
+                    self._max_col_samples_used, 
+                    other_profile._max_col_samples_used),
+                "column_count": utils.find_diff_of_numbers(
+                    len(self._profile), len(other_profile._profile)),
+                "row_count": utils.find_diff_of_numbers(
+                    self.total_samples, other_profile.total_samples),
+                "row_has_null_ratio": utils.find_diff_of_numbers(
+                    self._get_row_has_null_ratio(), 
+                    other_profile._get_row_has_null_ratio()),
+                "row_is_null_ratio": utils.find_diff_of_numbers(
+                    self._get_row_is_null_ratio(),
+                    other_profile._get_row_is_null_ratio()),
+                "unique_row_ratio": utils.find_diff_of_numbers(
+                    self._get_unique_row_ratio(),
+                    other_profile._get_unique_row_ratio()),
+                "duplicate_row_count": utils.find_diff_of_numbers(
+                    self._get_duplicate_row_count(),
+                    other_profile._get_row_is_null_ratio()),
+                "correlation_matrix": utils.find_diff_of_matrices(
+                    self.correlation_matrix, 
+                    other_profile.correlation_matrix),
+                "profile_schema": defaultdict(list)})
+        report.update({"data_stats": []})
+
+        # Extract the schema of each profile
+        self_profile_schema = defaultdict(list)
+        other_profile_schema = defaultdict(list)
+        for i in range(len(self._profile)):
+            col_name = self._profile[i].name
+            self_profile_schema[col_name].append(i)
+        for i in range(len(other_profile._profile)):
+            col_name = other_profile._profile[i].name
+            other_profile_schema[col_name].append(i)
+
+        report["global_stats"]["profile_schema"] = \
+            utils.find_diff_of_dicts_with_diff_keys(self_profile_schema, 
+                                                    other_profile_schema)
+
+        # Only find the diff of columns if the schemas are exactly the same
+        if self_profile_schema == other_profile_schema:
+            for i in range(len(self._profile)):
+                report["data_stats"].append(
+                    self._profile[i].diff(other_profile._profile[i], 
+                                          options=options))
+
+        return _prepare_report(report)
+
     @property
     def _max_col_samples_used(self):
         """
@@ -1331,7 +1547,8 @@ class StructuredProfiler(BaseProfiler):
                 "file_type": self.file_type,
                 "encoding": self.encoding,
                 "correlation_matrix": self.correlation_matrix,
-                "profile_schema": defaultdict(list)
+                "profile_schema": defaultdict(list),
+                "times": self.times,
             }),
             ("data_stats", []),
         ])
@@ -1348,19 +1565,24 @@ class StructuredProfiler(BaseProfiler):
         return _prepare_report(report, output_format, omit_keys)
 
     def _get_unique_row_ratio(self):
-        return len(self.hashed_row_dict) / self.total_samples
+        if self.total_samples:
+            return len(self.hashed_row_dict) / self.total_samples
+        return 0
 
     def _get_row_is_null_ratio(self):
-        return 0 if self._min_col_samples_used in {0, None} \
-            else self.row_is_null_count / self._min_col_samples_used
+        if self._min_col_samples_used:
+            return self.row_is_null_count / self._min_col_samples_used
+        return 0
 
     def _get_row_has_null_ratio(self):
-        return 0 if self._min_col_samples_used in {0, None} \
-            else self.row_has_null_count / self._min_col_samples_used
+        if self._min_col_samples_used:
+            return self.row_has_null_count / self._min_col_samples_used
+        return 0
 
     def _get_duplicate_row_count(self):
         return self.total_samples - len(self.hashed_row_dict)
 
+    @utils.method_timeit(name='row_stats')
     def _update_row_statistics(self, data, sample_ids=None):
         """
         Iterate over the provided dataset row by row and calculate
@@ -1431,25 +1653,40 @@ class StructuredProfiler(BaseProfiler):
             self.row_has_null_count = len(null_in_row_count)
             self.row_is_null_count = len(null_rows)
 
-    def _get_correlation(self, clean_samples):
+    def _get_correlation(self, clean_samples, batch_properties):
         """
         Calculate correlation matrix on the cleaned data.
 
         :param clean_samples: the input cleaned dataset
         :type clean_samples: dict()
+        :param batch_properties: mean/std/counts of each batch column necessary
+        for correlation computation
+        :type batch_properties: dict()
         """
         columns = self.options.correlation.columns
         clean_column_ids = []
         if columns is None:
             for idx in range(len(self._profile)):
-                if (self._profile[idx].profile['data_type'] not in ['int', 'float']
-                        or self._profile[idx].null_count > 0):
+                data_type = self._profile[idx].\
+                    profiles["data_type_profile"].selected_data_type
+                if data_type not in ["int", "float"]:
                     clean_samples.pop(idx)
                 else:
                     clean_column_ids.append(idx)
 
-        data = pd.DataFrame(clean_samples)
-        data = data.apply(pd.to_numeric, errors='coerce')
+        data = pd.DataFrame(clean_samples).apply(pd.to_numeric, errors='coerce')
+        means = {index:mean for index, mean in enumerate(batch_properties['mean'])}
+        data = data.fillna(value=means)
+
+        # Update the counts/std if needed (i.e. if null rows or exist)
+        if (len(data) != batch_properties['count']).any():
+            adjusted_stds = np.sqrt(
+                batch_properties['std']**2 * (batch_properties['count'] - 1) \
+                / (len(data) - 1)
+            )
+            batch_properties['std'] = adjusted_stds
+        # Set count key to a single number now that everything's been adjusted
+        batch_properties['count'] = len(data)
 
         # fill correlation matrix with nan initially
         n_cols = len(self._profile)
@@ -1458,8 +1695,10 @@ class StructuredProfiler(BaseProfiler):
         # then, fill in the correlations for valid columns
         rows = [[id] for id in clean_column_ids]
         corr_mat[rows, clean_column_ids] = np.corrcoef(data, rowvar=False)
+
         return corr_mat
 
+    @utils.method_timeit(name='correlation')
     def _update_correlation(self, clean_samples, prev_dependent_properties):
         """
         Update correlation matrix for cleaned data.
@@ -1467,18 +1706,16 @@ class StructuredProfiler(BaseProfiler):
         :param clean_samples: the input cleaned dataset
         :type clean_samples: dict()
         """
-        batch_corr = self._get_correlation(clean_samples)
-        batch_samples = np.nan
-        if len(clean_samples) > 0:
-            batch_samples = len(list(clean_samples.values())[0])
         batch_properties = self._get_correlation_dependent_properties(clean_samples)
+        batch_corr = self._get_correlation(clean_samples, batch_properties)
 
         self.correlation_matrix = self._merge_correlation_helper(
             self.correlation_matrix, prev_dependent_properties["mean"],
-            prev_dependent_properties["std"], self.total_samples,
+            prev_dependent_properties["std"], self.total_samples - self.row_is_null_count,
             batch_corr, batch_properties["mean"],
-            batch_properties["std"], batch_samples)
+            batch_properties["std"], batch_properties['count'])
 
+    @utils.method_timeit(name='correlation')
     def _merge_correlation(self, other):
         """
         Merge correlation matrix from two profiles
@@ -1488,9 +1725,11 @@ class StructuredProfiler(BaseProfiler):
         """
         corr_mat1 = self.correlation_matrix
         corr_mat2 = other.correlation_matrix
-        if self.total_samples == 0:
+        n1 = self.total_samples - self.row_is_null_count
+        n2 = other.total_samples - other.row_is_null_count
+        if n1 == 0:
             return corr_mat2
-        if other.total_samples == 0:
+        if n2 == 0:
             return corr_mat1
 
         if corr_mat1 is None or corr_mat2 is None:
@@ -1511,7 +1750,6 @@ class StructuredProfiler(BaseProfiler):
         std1 = np.array(
             [self._profile[idx].profile['statistics']['stddev']
              for idx in range(len(self._profile)) if idx in col_ids1])
-        n1 = self.total_samples
 
         mean2 = np.array(
             [other._profile[idx].profile['statistics']['mean']
@@ -1519,7 +1757,6 @@ class StructuredProfiler(BaseProfiler):
         std2 = np.array(
             [other._profile[idx].profile['statistics']['stddev']
              for idx in range(len(self._profile)) if idx in col_ids2])
-        n2 = other.total_samples
         return self._merge_correlation_helper(corr_mat1, mean1, std1, n1,
                                               corr_mat2, mean2, std2, n2)
 
@@ -1538,25 +1775,32 @@ class StructuredProfiler(BaseProfiler):
         """
         dependent_properties = {
             'mean': np.full(len(self._profile), np.nan),
-            'std': np.full(len(self._profile), np.nan)
+            'std': np.full(len(self._profile), np.nan),
+            'count': np.full(len(self._profile), np.nan)
         }
         for id in range(len(self._profile)):
-
-            data_type_compiler = self._profile[id].profiles["data_type_profile"]
+            compiler = self._profile[id]
+            data_type_compiler = compiler.profiles["data_type_profile"]
             data_type = data_type_compiler.selected_data_type
             if data_type in ["int", "float"]:
                 data_type_profiler = data_type_compiler._profiles[data_type]
+                # Finding dependent values of previous, existing data
                 if batch is None:
                     n = data_type_profiler.match_count
                     dependent_properties['mean'][id] = data_type_profiler.mean
+                    # Subtract null row count as those aren't included in corr. calc
                     dependent_properties['std'][id] = \
-                        np.sqrt(data_type_profiler._biased_variance * n / (n - 1))
+                        np.sqrt(data_type_profiler._biased_variance * n / (self.total_samples - self.row_is_null_count- 1))
+                    dependent_properties['count'][id] = n
+                # Finding the properties of the batch data if given
                 elif id in batch.keys():
                     history = data_type_profiler._batch_history[-1]
                     n = history['match_count']
+                    # Since we impute values, we want the total rows (including nulls)
                     dependent_properties['mean'][id] = history['mean']
                     dependent_properties['std'][id] = \
                         np.sqrt(history['biased_variance'] * n / (n - 1))
+                    dependent_properties['count'][id] = n
 
         return dependent_properties
 
@@ -1630,7 +1874,7 @@ class StructuredProfiler(BaseProfiler):
         if isinstance(data, pd.Series):
             data = data.to_frame()
         elif isinstance(data, list):
-            data = pd.DataFrame(data)
+            data = pd.DataFrame(data, dtype=object)
 
         # Calculate schema of incoming data
         mapping_given = defaultdict(list)
@@ -1647,11 +1891,17 @@ class StructuredProfiler(BaseProfiler):
 
         try:
             from tqdm import tqdm
+            has_tqdm = True
         except ImportError:
-            def tqdm(l):
-                for i, e in enumerate(l):
-                    print("Processing Column {}/{}".format(i + 1, len(l)))
-                    yield e
+            has_tqdm = False
+        finally:
+            if not has_tqdm or logger.getEffectiveLevel() > logging.INFO:
+                def tqdm(l):
+                    for i, e in enumerate(l):
+                        # These will automatically be ignored if user sets
+                        # logger level as higher than INFO
+                        logger.info(f"Processing Column {i + 1}/{len(l)}")
+                        yield e
 
         # Shuffle indices once and share with columns
         sample_ids = [*utils.shuffle_in_chunks(len(data), len(data))]
@@ -1696,7 +1946,7 @@ class StructuredProfiler(BaseProfiler):
                 cols=len(data.columns))
 
         # Format the data
-        notification_str = "Finding the Null values in the columns..."
+        notification_str = "Finding the Null values in the columns... "
         if pool:
             notification_str += " (with " + str(pool_size) + " processes)"
 
@@ -1719,16 +1969,17 @@ class StructuredProfiler(BaseProfiler):
                 if min_true_samples is None:
                     min_true_samples = self._profile[prof_idx]._min_true_samples
                 try:
+                    null_values = self._profile[prof_idx]._null_values
                     multi_process_dict[col_idx] = pool.apply_async(
                         self._profile[prof_idx].clean_data_and_get_base_stats,
-                        (col_ser, sample_size, min_true_samples,
+                        (col_ser, sample_size, null_values, min_true_samples,
                          sample_ids))
                 except Exception as e:
-                    print(e)
+                    logger.info(e)
                     single_process_list.add(col_idx)
 
             # Iterate through multiprocessed columns collecting results
-            print(notification_str)
+            logger.info(notification_str)
             for col_idx in tqdm(multi_process_dict.keys()):
                 try:
                     prof_idx = col_idx_to_prof_idx[col_idx]
@@ -1736,12 +1987,12 @@ class StructuredProfiler(BaseProfiler):
                         multi_process_dict[col_idx].get()
                     self._profile[prof_idx]._update_base_stats(base_stats)
                 except Exception as e:
-                    print(e)
+                    logger.info(e)
                     single_process_list.add(col_idx)
 
             # Clean up any columns which errored
             if len(single_process_list) > 0:
-                print("Errors in multiprocessing occured:",
+                logger.info("Errors in multiprocessing occured:",
                       len(single_process_list), "errors, reprocessing...")
                 for col_idx in tqdm(single_process_list):
                     col_ser = data.iloc[:, col_idx]
@@ -1749,9 +2000,10 @@ class StructuredProfiler(BaseProfiler):
                     if min_true_samples is None:
                         min_true_samples = \
                             self._profile[prof_idx]._min_true_samples
+                    null_values = self._profile[prof_idx]._null_values
                     clean_sampled_dict[prof_idx], base_stats = \
                         self._profile[prof_idx].clean_data_and_get_base_stats(
-                            col_ser, sample_size,
+                            col_ser, sample_size, null_values,
                             min_true_samples, sample_ids)
                     self._profile[prof_idx]._update_base_stats(base_stats)
 
@@ -1759,15 +2011,17 @@ class StructuredProfiler(BaseProfiler):
             pool.join()  # Wait for all workers to complete
 
         else:  # No pool
-            print(notification_str)
+            logger.info(notification_str)
             for col_idx in tqdm(range(data.shape[1])):
                 col_ser = data.iloc[:, col_idx]
                 prof_idx = col_idx_to_prof_idx[col_idx]
                 if min_true_samples is None:
                     min_true_samples = self._profile[prof_idx]._min_true_samples
+                null_values = self._profile[prof_idx]._null_values
                 clean_sampled_dict[prof_idx], base_stats = \
                     self._profile[prof_idx].clean_data_and_get_base_stats(
                         df_series=col_ser, sample_size=sample_size,
+                        null_values=null_values,
                         min_true_samples=min_true_samples,
                         sample_ids=sample_ids
                     )
@@ -1781,7 +2035,7 @@ class StructuredProfiler(BaseProfiler):
             if pool:
                 notification_str += " (with " + str(pool_size) + " processes)"
 
-        print(notification_str)
+        logger.info(notification_str)
 
         for prof_idx in tqdm(clean_sampled_dict.keys()):
             self._profile[prof_idx].update_column_profilers(
@@ -1797,7 +2051,8 @@ class StructuredProfiler(BaseProfiler):
             samples_for_row_stats = np.concatenate(sample_ids)
 
         if self.options.correlation.is_enabled:
-            self._update_correlation(clean_sampled_dict, corr_prev_dependent_properties)
+            self._update_correlation(clean_sampled_dict,
+                                     corr_prev_dependent_properties)
         self._update_row_statistics(data, samples_for_row_stats)
 
     def save(self, filepath=None):
@@ -1821,6 +2076,7 @@ class StructuredProfiler(BaseProfiler):
             "options": self.options,
             "_profile": self.profile,
             "_col_name_to_idx": self._col_name_to_idx,
+            "times": self.times,
         }
 
         self._save_helper(filepath, data_dict)
