@@ -3,6 +3,7 @@
 coding=utf-8
 Specify the options when running the data profiler.
 """
+import re
 import warnings
 import abc
 import copy
@@ -209,6 +210,36 @@ class HistogramOption(BooleanOption):
                                                       valid_methods))
         return errors
 
+class ModeOption(BooleanOption):
+
+    def __init__(self, is_enabled=True, max_k_modes=5):
+        """Options for mode estimation
+
+        :ivar is_enabled: boolean option to enable/disable the option.
+        :vartype is_enabled: bool
+        :ivar top_k_modes: the max number of modes to return, if applicable
+        :vartype top_k_modes: int
+        """
+        self.top_k_modes = max_k_modes
+        super().__init__(is_enabled=is_enabled)
+
+    def _validate_helper(self, variable_path='ModeOption'):
+        """
+        Validates the options do not conflict and cause errors.
+
+        :param variable_path: current path to variable set.
+        :type variable_path: str
+        :return: list of errors (if raise_error is false)
+        :rtype: list(str)
+        """
+        errors = super()._validate_helper(variable_path=variable_path)
+
+        if self.top_k_modes is not None and (
+                not isinstance(self.top_k_modes, int)
+                or self.top_k_modes < 1):
+            errors.append("{}.top_k_modes must be either None"
+                          " or a positive integer".format(variable_path))
+        return errors
 
 class BaseInspectorOptions(BooleanOption):
 
@@ -265,6 +296,8 @@ class NumericalOptions(BaseInspectorOptions):
         :vartype min: BooleanOption
         :ivar max: boolean option to enable/disable max
         :vartype max: BooleanOption
+        :ivar mode: option to enable/disable mode and set return count
+        :vartype mode: ModeOption
         :ivar sum: boolean option to enable/disable sum
         :vartype sum: BooleanOption
         :ivar variance: boolean option to enable/disable variance
@@ -288,6 +321,7 @@ class NumericalOptions(BaseInspectorOptions):
         """
         self.min = BooleanOption(is_enabled=True)
         self.max = BooleanOption(is_enabled=True)
+        self.mode = ModeOption(is_enabled=True)
         self.sum = BooleanOption(is_enabled=True)
         self.variance = BooleanOption(is_enabled=True)
         self.skewness = BooleanOption(is_enabled=True)
@@ -309,7 +343,8 @@ class NumericalOptions(BaseInspectorOptions):
         :return: true if any numeric stats property is enabled, otherwise false
         :rtype bool:
         """
-        if self.min.is_enabled or self.max.is_enabled or self.sum.is_enabled \
+        if self.min.is_enabled or self.max.is_enabled \
+                or self.mode.is_enabled or self.sum.is_enabled \
                 or self.variance.is_enabled or self.skewness.is_enabled \
                 or self.kurtosis.is_enabled \
                 or self.histogram_and_quantiles.is_enabled \
@@ -330,6 +365,7 @@ class NumericalOptions(BaseInspectorOptions):
         """
         self.min.is_enabled = value
         self.max.is_enabled = value
+        self.mode.is_enabled = value
         self.sum.is_enabled = value
         self.variance.is_enabled = value
         self.skewness.is_enabled = value
@@ -361,7 +397,7 @@ class NumericalOptions(BaseInspectorOptions):
             variable_path = self.__class__.__name__
 
         errors = super()._validate_helper(variable_path=variable_path)
-        for item in ["histogram_and_quantiles", "min", "max", "sum",
+        for item in ["histogram_and_quantiles", "min", "max", "sum", "mode",
                      "variance", "skewness", "kurtosis", "bias_correction",
                      "num_zeros", "num_negatives"]:
             if not isinstance(self.properties[item], BooleanOption):
@@ -391,6 +427,14 @@ class NumericalOptions(BaseInspectorOptions):
                           " variance, and skewness if kurtosis is "
                           "toggled on.".format(variable_path))
 
+        mode_disabled = not self.properties["mode"].is_enabled
+        histogram_disabled = \
+            not self.properties["histogram_and_quantiles"].is_enabled
+        if histogram_disabled and not mode_disabled:
+            errors.append("{}: The numeric stats must toggle on histogram "
+                          "and quantiles if mode is "
+                          "toggled on.".format(variable_path))
+
         # warn user if all stats are disabled
         if not errors:
             if not self.is_numeric_stats_enabled:
@@ -414,6 +458,8 @@ class IntOptions(NumericalOptions):
         :vartype min: BooleanOption
         :ivar max: boolean option to enable/disable max
         :vartype max: BooleanOption
+        :ivar mode: option to enable/disable mode and set return count
+        :vartype mode: ModeOption
         :ivar sum: boolean option to enable/disable sum
         :vartype sum: BooleanOption
         :ivar variance: boolean option to enable/disable variance
@@ -501,6 +547,8 @@ class FloatOptions(NumericalOptions):
         :vartype min: BooleanOption
         :ivar max: boolean option to enable/disable max
         :vartype max: BooleanOption
+        :ivar mode: option to enable/disable mode and set return count
+        :vartype mode: ModeOption
         :ivar sum: boolean option to enable/disable sum
         :vartype sum: BooleanOption
         :ivar variance: boolean option to enable/disable variance
@@ -556,6 +604,8 @@ class TextOptions(NumericalOptions):
         :vartype min: BooleanOption
         :ivar max: boolean option to enable/disable max
         :vartype max: BooleanOption
+        :ivar mode: option to enable/disable mode and set return count
+        :vartype mode: ModeOption
         :ivar sum: boolean option to enable/disable sum
         :vartype sum: BooleanOption
         :ivar variance: boolean option to enable/disable variance
@@ -622,7 +672,8 @@ class TextOptions(NumericalOptions):
         :return: true if any numeric stats property is enabled, otherwise false
         :rtype bool:
         """
-        if self.min.is_enabled or self.max.is_enabled or self.sum.is_enabled \
+        if self.min.is_enabled or self.max.is_enabled \
+                or self.sum.is_enabled or self.mode.is_enabled \
                 or self.variance.is_enabled or self.skewness.is_enabled \
                 or self.kurtosis.is_enabled \
                 or self.histogram_and_quantiles.is_enabled:
@@ -641,6 +692,7 @@ class TextOptions(NumericalOptions):
         """
         self.min.is_enabled = value
         self.max.is_enabled = value
+        self.mode.is_enabled = value
         self.sum.is_enabled = value
         self.variance.is_enabled = value
         self.skewness.is_enabled = value
@@ -701,6 +753,8 @@ class CategoricalOptions(BaseInspectorOptions):
 
         :ivar is_enabled: boolean option to enable/disable the column.
         :vartype is_enabled: bool
+        :ivar top_k_categories: number of categories to be displayed when called
+        :vartype top_k_categories: [None, int]
         """
         BaseInspectorOptions.__init__(self, is_enabled=is_enabled)
         self.top_k_categories = top_k_categories
@@ -723,7 +777,6 @@ class CategoricalOptions(BaseInspectorOptions):
         return errors
 
 
-
 class CorrelationOptions(BaseInspectorOptions):
 
     def __init__(self, is_enabled=False, columns=None):
@@ -927,10 +980,12 @@ class TextProfilerOptions(BaseInspectorOptions):
 
 class StructuredOptions(BaseOption):
 
-    def __init__(self):
+    def __init__(self, null_values=None):
         """
         Constructs the StructuredOptions object with default values.
 
+        :param null_values: null values we input.
+        :vartype null_values: Union[None, dict]
         :ivar int: option set for int profiling.
         :vartype int: IntOptions
         :ivar float: option set for float profiling.
@@ -947,7 +1002,10 @@ class StructuredOptions(BaseOption):
         :vartype data_labeler: DataLabelerOptions
         :ivar correlation: option set for correlation profiling.
         :vartype correlation: CorrelationOptions
+        :ivar null_values: option set for defined null values
+        :vartype null_values: Union[None, dict]
         """
+        # Option variables
         self.multiprocess = BooleanOption()
         self.int = IntOptions()
         self.float = FloatOptions()
@@ -957,12 +1015,17 @@ class StructuredOptions(BaseOption):
         self.category = CategoricalOptions()
         self.data_labeler = DataLabelerOptions()
         self.correlation = CorrelationOptions()
+        # Non-Option variables
+        self.null_values = null_values
 
     @property
     def enabled_profiles(self):
         """Returns a list of the enabled profilers for columns."""
         enabled_profiles = list()
-        for key, value in self.properties.items():
+        # null_values does not have is_enabled
+        properties = self.properties
+        properties.pop('null_values')
+        for key, value in properties.items():
             if value.is_enabled:
                 enabled_profiles.append(key)
         return enabled_profiles
@@ -992,8 +1055,9 @@ class StructuredOptions(BaseOption):
             ('data_labeler', DataLabelerOptions),
             ('correlation', CorrelationOptions)
         ])
-
-        for column in self.properties:
+        properties = self.properties
+        properties.pop('null_values')
+        for column in (properties):
             if not isinstance(self.properties[column], prop_check[column]):
                 errors.append("{}.{} must be a(n) {}.".format(
                     variable_path, column, prop_check[column].__name__))
@@ -1001,6 +1065,17 @@ class StructuredOptions(BaseOption):
                 errors += self.properties[column]._validate_helper(
                     variable_path=(variable_path + '.' + column
                                    if variable_path else column))
+
+        if self.null_values is not None and not (
+                isinstance(self.null_values, dict) and
+                all(isinstance(key, str) and
+                    (isinstance(value, re.RegexFlag) or value == 0)
+                    for key, value in self.null_values.items())):
+            errors.append("{}.null_values must be either None or "
+                          "a dictionary that contains keys of type str "
+                          "and values == 0 or are instances of "
+                          "a re.RegexFlag".format(variable_path))
+
         return errors
 
 
