@@ -45,34 +45,41 @@ class TestJSONDataClass(unittest.TestCase):
             dict(path=os.path.join(test_dir, "json/honeypot_intentially_mislabeled_file.csv"), encoding='utf-8', count=14),
             dict(path=os.path.join(test_dir, "json/honeypot_intentially_mislabeled_file.parquet"), encoding='utf-8', count=14)
         ]
-    
-    def test_is_match_for_string_streams(self):
-        """
-        Determine if the json file can be automatically identified from
-        string stream
-        """
-        for input_file in self.input_file_names:
-            print(input_file)
-            with open(input_file['path'], 'r',
-                      encoding=input_file['encoding']) as fp:
-                byte_string = StringIO(fp.read())
-                self.assertTrue(JSONData.is_match(byte_string))
 
-    def test_is_match_for_byte_streams(self):
+        cls.buffer_list = []
+        for input_file in cls.input_file_names:
+            # add StringIO
+            buffer_info = input_file.copy()
+            with open(input_file['path'], 'r', encoding=input_file['encoding']) as fp:
+                buffer_info['path'] = StringIO(fp.read())
+            cls.buffer_list.append(buffer_info)
+            
+            # add BytesIO
+            buffer_info = input_file.copy()
+            with open(input_file['path'], 'rb') as fp:
+                buffer_info['path'] = BytesIO(fp.read())
+            cls.buffer_list.append(buffer_info)
+
+        cls.file_or_buf_list = cls.input_file_names + cls.buffer_list
+
+    @classmethod
+    def setUp(cls):
+        for buffer in cls.buffer_list:
+            buffer['path'].seek(0)
+
+    def test_is_match(self):
         """
         Determine if the json file can be automatically identified from
-        byte stream
+        byte stream or stringio stream or filepath
         """
-        for input_file in self.input_file_names:
-            with open(input_file['path'], 'rb') as fp:
-                byte_string = BytesIO(fp.read())
-                self.assertTrue(JSONData.is_match(byte_string))
+        for input_file in self.file_or_buf_list:
+            self.assertTrue(JSONData.is_match(input_file['path']))
 
     def test_json_file_identification(self):
         """
         Determine if the json file can be automatically identified
         """
-        for input_file in self.input_file_names:
+        for input_file in self.file_or_buf_list:
             input_data_obj = Data(input_file["path"])
             self.assertEqual(input_data_obj.data_type, 'json')
 
@@ -80,7 +87,7 @@ class TestJSONDataClass(unittest.TestCase):
         """
         Determine if the json file can be loaded with manual data_type setting
         """
-        for input_file in self.input_file_names:
+        for input_file in self.file_or_buf_list:
             input_data_obj = Data(input_file["path"], data_type='json')
             self.assertEqual(input_data_obj.data_type, 'json')
 
@@ -88,7 +95,7 @@ class TestJSONDataClass(unittest.TestCase):
         """
         Determine if the json file can be reloaded
         """
-        for input_file in self.input_file_names:
+        for input_file in self.file_or_buf_list:
             input_data_obj = Data(input_file["path"])
             input_data_obj.reload(input_file["path"])
             self.assertEqual(input_data_obj.data_type, 'json')
@@ -132,7 +139,7 @@ class TestJSONDataClass(unittest.TestCase):
         """
         Determine if the json file data_formats can be used
         """
-        for input_file in self.input_file_names:
+        for input_file in self.file_or_buf_list:
             input_data_obj = Data(input_file["path"])
             for data_format in list(input_data_obj._data_formats.keys()):
                 input_data_obj.data_format = data_format
@@ -151,7 +158,7 @@ class TestJSONDataClass(unittest.TestCase):
         length value.
         """
 
-        for input_file in self.input_file_names:
+        for input_file in self.file_or_buf_list:
             data = Data(input_file["path"])
             self.assertEqual(input_file['count'],
                              len(data),
@@ -203,7 +210,6 @@ class TestJSONDataClass(unittest.TestCase):
         self.assertEqual(3, len(complex.metadata.columns))
         self.assertEqual("Frodo", complex.data_and_metadata["meta.creator"][0])
 
-
     def test_list_of_dictionaries_in_flattened_dataframe_format(self):
         test_dir = os.path.join(test_root_path, 'data')
         input_file_name = os.path.join(test_dir, 'json/iris-utf-8.json')
