@@ -1,20 +1,25 @@
+import re
+
 from dataprofiler.profilers.profiler_options import StructuredOptions
 from dataprofiler.tests.profilers.profiler_options.test_base_option \
      import TestBaseOption
 
 
 class TestStructuredOptions(TestBaseOption):
-    
+
     option_class = StructuredOptions
-    keys = ["int", "float", "datetime", "text", "order", "category",
-            "data_labeler", "multiprocess", "correlation"]
+    other_keys = ["null_values"]
+    boolean_keys = ["int", "float", "datetime", "text", "order", "category",
+                    "data_labeler", "multiprocess", "correlation"]
+    keys = boolean_keys + other_keys
+
 
     @classmethod
     def get_options(self, **params):
         options = StructuredOptions()
         options.set(params)
         return options
-    
+
     def test_init(self):
         options = self.get_options()
         for key in self.keys:
@@ -24,41 +29,61 @@ class TestStructuredOptions(TestBaseOption):
         super().test_set_helper()
         option = self.get_options()
         optpth = self.get_options_path()
-        
         # Enable and Disable Option
-        for key in self.keys:
+        for key in self.boolean_keys:
             option._set_helper({'{}.is_enabled'.format(key): False}, '')
-            self.assertFalse(option.properties[key].is_enabled)        
+            self.assertFalse(option.properties[key].is_enabled)
             option._set_helper({'{}.is_enabled'.format(key): True}, '')
-            self.assertTrue(option.properties[key].is_enabled)        
+            self.assertTrue(option.properties[key].is_enabled)
 
         # Treat is_enabled as a BooleanOption
-        for key in self.keys:
+        for key in self.boolean_keys:
             expected_error = "type object '{}.is_enabled' has no attribute " \
                              "'is_enabled'".format(key)
             with self.assertRaisesRegex(AttributeError, expected_error):
                 option._set_helper({'{}.is_enabled.is_enabled' \
                                    .format(key): True}, '')
-        
+
     def test_set(self):
         super().test_set()
         option = self.get_options()
         optpth = self.get_options_path()
 
         # Enable and Disable Options        
-        for key in self.keys:
+        for key in self.boolean_keys:
             option.set({'{}.is_enabled'.format(key): False})
-            self.assertFalse(option.properties[key].is_enabled)        
+            self.assertFalse(option.properties[key].is_enabled)
             option.set({'{}.is_enabled'.format(key): True})
-            self.assertTrue(option.properties[key].is_enabled)        
-    
+            self.assertTrue(option.properties[key].is_enabled)
+
         # Treat is_enabled as a BooleanOption
-        for key in self.keys:
+        for key in self.boolean_keys:
             expected_error = "type object '{}.is_enabled' has no attribute " \
                              "'is_enabled'".format(key)
             with self.assertRaisesRegex(AttributeError, expected_error):
                 option.set({'{}.is_enabled.is_enabled'.format(key): True})
-    
+
+        for key in self.other_keys:
+            expected_error = "type object '{}' has no attribute " \
+                             "'is_enabled'".format(key)
+            with self.assertRaisesRegex(AttributeError, expected_error):
+                option.set({'{}.is_enabled'.format(key): True})
+
+        expected_error = "{}.null_values must be either None or " \
+                              "a dictionary that contains keys of str type " \
+                              "and values == 0 or are instances of " \
+                              "a re.RegexFlag".format(optpth)
+
+        test_dict = {'a': 0}
+        option.set({'null_values': test_dict})
+        self.assertEqual({'a': 0}, option.null_values)
+        test_dict = {'a': re.IGNORECASE}
+        option.set({'null_values': test_dict})
+        self.assertEqual({'a': 2}, option.null_values)
+        test_dict = None
+        option.set({'null_values': test_dict})
+        self.assertEqual(None, option.null_values)
+
     def test_validate_helper(self):
         # Valid cases should return [] while invalid cases
         # should return a list of errors
@@ -67,17 +92,17 @@ class TestStructuredOptions(TestBaseOption):
 
         # Default Configuration Is Valid
         self.assertEqual([], option._validate_helper())
-        
+
         # Variable Path Is Not A String
         expected_error = "The variable path must be a string."
         with self.assertRaisesRegex(ValueError, expected_error):
             option._validate_helper(1)
-        
+
         # Option is_enabled is not a boolean
-        for key in self.keys:
-            option.set({'{}.is_enabled'.format(key): "Hello World"}) 
+        for key in self.boolean_keys:
+            option.set({'{}.is_enabled'.format(key): "Hello World"})
         expected_error = ['{}.{}.is_enabled must be a Boolean.' \
-                          .format(optpth, key) for key in self.keys]
+                          .format(optpth, key) for key in self.boolean_keys]
         expected_error = set(expected_error)
         # Verify expected errors are a subset of all errors
         self.assertSetEqual(expected_error,
@@ -97,7 +122,7 @@ class TestStructuredOptions(TestBaseOption):
         option.correlation = StructuredOptions()
 
         expected_error = set()
-        for key in self.keys:
+        for key in self.boolean_keys:
             ckey = key.capitalize()
             if key == "data_labeler": ckey = "DataLabeler"
             elif key == "category": ckey = "Categorical"
@@ -109,23 +134,23 @@ class TestStructuredOptions(TestBaseOption):
                 expected_error.add('{}.{} must be a(n) {}Options.' \
                                    .format(optpth, key, ckey))
         self.assertSetEqual(expected_error, set(option._validate_helper()))
-            
+
     def test_validate(self):
         # Valid cases should return None while invalid cases
         # should return or throw a list of errors
         option = self.get_options()
         optpth = self.get_options_path()
-    
+
         # Default Configuration Is Valid
         self.assertEqual(None, option.validate())
-        
+
         # Option is_enabled is not a boolean
-        for key in self.keys:
-            option.set({'{}.is_enabled'.format(key): "Hello World"}) 
-        
+        for key in self.boolean_keys:
+            option.set({'{}.is_enabled'.format(key): "Hello World"})
+
         expected_error = ["{}.{}.is_enabled must be a Boolean." \
                               .format(optpth, key)
-                          for key in self.keys]
+                          for key in self.boolean_keys]
         expected_error = set(expected_error)
         # Verify expected errors are a subset of all errors
         with self.assertRaises(ValueError) as cm:
@@ -151,7 +176,7 @@ class TestStructuredOptions(TestBaseOption):
         option.correlation = StructuredOptions()
 
         expected_error = set()
-        for key in self.keys:
+        for key in self.boolean_keys:
             ckey = key.capitalize()
             if key == "data_labeler": ckey = "DataLabeler"
             elif key == "category": ckey = "Categorical"
@@ -169,22 +194,47 @@ class TestStructuredOptions(TestBaseOption):
             option.validate(raise_error=True)
         raised_error = set(str(cm.exception).split("\n"))
         self.assertEqual(expected_error, raised_error)
-            
+        option = self.get_options()
+        expected_error = ['{}.null_values must be either None or a '
+                          'dictionary that contains keys of type str '
+                          'and values == 0 or are instances of '
+                          'a re.RegexFlag'.format(optpth)]
+        # Test key is not a string
+        option.set({'null_values': {0: 0}})
+        self.assertEqual(expected_error, option._validate_helper())
+        # Test value is not correct type (0 or regex)
+        option.set({'null_values': {"a": 1}})
+        self.assertEqual(expected_error, option._validate_helper())
+        # Test variable is not correct variable type
+        option.set({'null_values': 1})
+        self.assertEqual(expected_error, option._validate_helper())
+        # Test 0 works for option set
+        option.set({'null_values': {"a": 0}})
+        self.assertEqual([], option._validate_helper())
+        # Test a regex flag works for option set
+        option.set({'null_values': {"a": re.IGNORECASE}})
+        self.assertEqual([], option._validate_helper())
+        # Test None works for option set
+        option.set({'null_values': None})
+        self.assertEqual([], option._validate_helper())
+
+
     def test_enabled_profilers(self):
         options = self.get_options()
-        
+        self.assertNotIn('null_values', options.enabled_profiles)
+
         # All Columns Enabled
-        for key in self.keys: 
+        for key in self.boolean_keys:
             options.set({'{}.is_enabled'.format(key): True})
-        self.assertSetEqual(set(self.keys), set(options.enabled_profiles))
+        self.assertSetEqual(set(self.boolean_keys), set(options.enabled_profiles))
 
         # No Columns Enabled        
-        for key in self.keys: 
+        for key in self.boolean_keys:
             options.set({'{}.is_enabled'.format(key): False})
         self.assertEqual([], options.enabled_profiles)
 
         # One Column Enabled
-        for key in self.keys:
+        for key in self.boolean_keys:
             options.set({'{}.is_enabled'.format(key): True})
             self.assertSetEqual(set([key]), set(options.enabled_profiles))
             options.set({'{}.is_enabled'.format(key): False})
