@@ -185,6 +185,7 @@ class TestTextColumnProfiler(unittest.TestCase):
         expected_profile = dict(
             min=1.0,
             max=4.0,
+            mode=[1],
             sum=20.0,
             mean=20.0 / 10.0,
             variance=14.0 / 9.0,
@@ -219,6 +220,11 @@ class TestTextColumnProfiler(unittest.TestCase):
             histogram = profile.pop('histogram')
             vocab = profile.pop('vocab')
 
+            # validate mode
+            expected_mode = expected_profile.pop('mode')
+            mode = profile.pop('mode')
+            np.testing.assert_array_almost_equal(expected_mode, mode, decimal=2)
+
             # key and value populated correctly
             self.assertDictEqual(expected_profile, profile)
             self.assertTrue(np.all(
@@ -444,9 +450,9 @@ class TestTextColumnProfiler(unittest.TestCase):
             'y', 'n', 'i', 's', 'G', 'r', 't', 'I', 'v', '6', '7', 'g'
         ]
 
-        profiler = TextColumn(df.name)
-        profiler.update(df)
-        profile1 = profiler.profile
+        profiler1 = TextColumn(df.name)
+        profiler1.update(df)
+        profile1 = profiler1.profile
 
         profiler2 = TextColumn(df2.name)
         profiler2.update(df2)
@@ -459,8 +465,18 @@ class TestTextColumnProfiler(unittest.TestCase):
                          'variance': profile1['variance'] - profile2['variance'],
                          'stddev': profile1['stddev'] - profiler2['stddev'],
                          'vocab': utils.find_diff_of_lists_and_sets(
-                             profile1['vocab'], profile2['vocab'])
+                             profile1['vocab'], profile2['vocab']),
+                         't-test': {
+                             't-statistic': -1.9339958714826413,
+                             'conservative': {
+                                 'df': 8,
+                                 'p-value': 0.08916903961929257
+                             },
+                             'welch': {
+                                 'df': 15.761400272034564,
+                                 'p-value': 0.07127621949432528
+                             }
                          }
-        diff = profiler.diff(profiler2)
+                         }
+        diff = profiler1.diff(profiler2)
         self.assertDictEqual(expected_diff, diff)
-        
